////////////////////////////////////////////////////////////
//
// Project:       Delayline Detector
//
// Module:        driver library (DLL)
// Date:          03.09.2004
// File:          DelayLine.h
//
// Description:   Defines the entry point for the DLL application
//
///////////////////////////////////////////////////////////
//
// 20.10.04  gf   GetRunning, SetImageSize
// 30.10.04  ao   SetCNT, GetCNT, ReadyCNT

#ifndef _DelayLine_h_
#define _DelayLine_h_


#ifdef DELAYLINE_EXPORTS
#define DELAYLINE_API __declspec(dllexport)
#else
#define DELAYLINE_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif  // __cplusplus

#include "DelayLineDefs.h"
static DldDev*			s_dld[]                   = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // max. 16 TDC devices



///////////////////////////////////////////////////////////
//
// initalisation methods
//

///////////////////////////////////////////////////////////
//
// DLD_OPEN
//
// initialize library and hardware
//
// input:
//          dld_id      :  empty pointer for driver id
//
// output:
//          dld_id      :  not NULL:   driver id, success
//                         NULL:       error
//
// return:  ERR_NO or error code
//           
DELAYLINE_API int DLD_OPEN (DLD_ID *dld_id);



///////////////////////////////////////////////////////////
//
// DLD_OPEN_EXT
//
// initialize library and hardware, extended
//
// input:
//          dld_id         :  empty pointer for driver id
//          DLD_DLL_MODE   :  dll mode (usb|dummy)
//          char*          :  dll config string (base of registry key)
//
// output:
//          dld_id      :  not NULL:   driver id, success
//                         NULL:       error
//
// return:  ERR_NO or error code
//           
DELAYLINE_API int DLD_OPEN_EXT (DLD_ID *dld_id, DLD_DLL_MODE dll_mode, char* config_str);



///////////////////////////////////////////////////////////
//
// DLD_CLOSE
//
// close library and hardware
//
// input:
//          dld_id      :  pointer with current driver id
//
// output:
//          dld_id      :  NULL     :   driver id reset, success
//                         not NULL :   error
//
// return:  ERR_NO or error code
//           
DELAYLINE_API int DLD_CLOSE (DLD_ID *dld_id);


///////////////////////////////////////////////////////////
//
// DLD_RESET
//
// reset library and hardware
//
// input:
//          dld_id      :  driver id
//
// output:
//
// return:  ERR_NO or error code
//           
DELAYLINE_API int DLD_RESET (DLD_ID dld_id);




///////////////////////////////////////////////////////////
//
// info requests
//

///////////////////////////////////////////////////////////
//
// DLD_GET_INFO_STRING
//
// input:
//          dld_id         :  driver id
//
//          info_str       :  empty pointer for device info description
//          len            :  max. size of info_str
//
// output:
//          info_string    :  device info description,
//                            e.g. "DelayLine detector V1.2"
//
// return:  ERR_NO or error code, e.g. 
//
DELAYLINE_API int DLD_GET_INFO_STRING (DLD_ID dld_id, char* info_str, int len);


///////////////////////////////////////////////////////////
//
// DLD_ERROR_STRING
//
// input:
//          error          :  error number
//
//          error_str      :  empty pointer for error description
//          len            :  max. size of error_str
//
// output:
//          err_string     :  error description
//
// return:  ERR_NO or error code, e.g. DLD_ERR_INVALID_ERRORCODE
//
DELAYLINE_API int DLD_ERROR_STRING (int error, char* error_str, int len);



///////////////////////////////////////////////////////////
//
// DLD_GET_DEVICE_TYPE
//
// input:
//          dld_id         :  driver id
//
//          dev_type       :  empty pointer for device type id
//
// output:
//          dev_type       :  device type id
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_DEVICE_TYPE (DLD_ID dld_id, int *dev_type);



///////////////////////////////////////////////////////////
//
// configuration requests
//


///////////////////////////////////////////////////////////
//
// DLD_GET_DEVICE_INFO
//
// input:
//          dld_id         :  driver id
//
//          properties     :  pointer to empty property struct
//
// output:
//          properties     :  filled property struct
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_DEVICE_INFO (DLD_ID dld_id, 
                   dld_properties* properties);



///////////////////////////////////////////////////////////
//
// DLD_GET_RUNNING
//
// input:
//          dld_id            :  driver id
//
//          running           :  pointer to result buffer
//
// output:
//          *running          :  if running or not
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_RUNNING (DLD_ID dld_id, int * running);



///////////////////////////////////////////////////////////
//
// DLD_SET_IMAGE_SIZE
//
// input:
//          dld_id            :  driver id
//
//          dld_image_size    :  requested size
//
// output:  -
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_IMAGE_SIZE (DLD_ID dld_id, dld_image_size size);



///////////////////////////////////////////////////////////
//
// DLD_GET_IMAGE_SIZE
//
// input:
//          dld_id            :  driver id
//
//          dld_image_size    :  pointer to empty image size struct
//
// output:
//          dld_image_size    :  filled image size struct
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_IMAGE_SIZE (DLD_ID dld_id, dld_image_size* size);



///////////////////////////////////////////////////////////
//
// DLD_GET_IMG_ROI
//
// input:
//          dld_id            :  driver id
//
//          dld_image_roi     :  pointer to empty image roi struct
//
// output:
//          dld_image_roi     :  filled image roi struct
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_IMG_ROI (DLD_ID dld_id, dld_image_roi* roi);

 

///////////////////////////////////////////////////////////
//
// DLD_SET_IMG_ROI
//
// input:
//          dld_id            :  driver id
//
//          dld_image_roi     :  image roi struct
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_IMG_ROI (DLD_ID dld_id, dld_image_roi roi);






///////////////////////////////////////////////////////////
//
// DLD_GET_IMG_BINNING_INFO
//
// input:
//          dld_id            :  driver id
//
//          bin_max_x         :  empty pointer
//          bin_max_y         :  empty pointer
//
// output:
//          bin_max_x         :  number of available binnings in x
//          bin_max_y         :  number of available binnings in y
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_IMG_BINNING_INFO_MAX (DLD_ID dld_id, 
                   int* bin_max_x, int* bin_max_y, int* bin_max_t);


///////////////////////////////////////////////////////////
//
// DLD_GET_IMG_BINNING_INFO
//
// input:
//          dld_id         :  driver id
//
//          binnings       :  pointer to empty binning info struct
//
// output:
//          binnings       :  filled binning info struct
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_IMG_BINNING_INFO (DLD_ID dld_id, 
                   dld_image_binnings* binnigs);



///////////////////////////////////////////////////////////
//
// DLD_GET_IMG_BINNING
//
// input:
//          dld_id         :  driver id
//
//          binning        :  pointer to empty binning struct
//
// output:
//          binning        :  filled binning struct
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_IMG_BINNING (DLD_ID dld_id, 
                   dld_image_binning* binnig);



///////////////////////////////////////////////////////////
//
// DLD_SET_IMG_BINNING
//
// input:
//          dld_id         :  driver id
//
//          binning        :  filled binning struct
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_IMG_BINNING (DLD_ID dld_id, 
                   dld_image_binning binnig);




///////////////////////////////////////////////////////////
//
// DLD_GET_DEV_PARAMS
//
// input:
//          dld_id         :  driver id
//
//          dld_dev_params :  pointer to empty device parameter struct
//
// output:
//          binning        :  filled binning struct
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_DEV_PARAMS (DLD_ID dld_id, 
                   dld_dev_params* dev_params);



///////////////////////////////////////////////////////////
//
// DLD_SET_DEV_PARAMS
//
// input:
//          dld_id         :  driver id
//
//          dld_dev_params :  filled device parameter struct
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_DEV_PARAMS (DLD_ID dld_id, 
                   dld_dev_params dev_params);




///////////////////////////////////////////////////////////
//
// DLD_GET_IMG_THRESHOLD
//
// input:
//          dld_id         :  driver id
//
//          threshold      :  empty pointer
//
// output:
//          threshold      :  delayline threshold
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_IMG_THRESHOLD (DLD_ID dld_id, 
                   double* threshold);



///////////////////////////////////////////////////////////
//
// DLD_SET_IMG_THRESHOLD
//
// input:
//          dld_id         :  driver id
//
//          threshold      :  delayline threshold
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_IMG_THRESHOLD (DLD_ID dld_id, 
                   double threshold);



///////////////////////////////////////////////////////////
//
// DLD_GET_CHANNEL_THRESHOLD
//
// input:
//          dld_id         :  driver id
//
//          threshold      :  empty pointer
//          len            :  number of channels
//
// output:
//          threshold      :  channel thresholds
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_CHANNEL_THRESHOLD (DLD_ID dld_id, 
                   double* threshold, int len);



///////////////////////////////////////////////////////////
//
// DLD_SET_CHANNEL_THRESHOLD
//
// input:
//          dld_id         :  driver id
//
//          threshold      :  channel thresholds
//          len            :  number of channels
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_CHANNEL_THRESHOLD (DLD_ID dld_id, 
                   double* threshold, int len);



///////////////////////////////////////////////////////////
//
// DLD_GET_EXPOSURE_TIME
//
// input:
//          dld_id         :  driver id
//
//          exposure       :  pointer to empty exposure struct
//
// output:
//          exposure       :  filled exposure struct
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_EXPOSURE_TIME (DLD_ID dld_id, 
                   dld_exposure* exposure);



///////////////////////////////////////////////////////////
//
// DLD_SET_EXPOSURE_TIME
//
// input:
//          dld_id         :  driver id
//
//          exposure       :  pointer to exposure struct
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_EXPOSURE_TIME (DLD_ID dld_id, 
                   dld_exposure exposure);



///////////////////////////////////////////////////////////
//
// DLD_GET_DATA_INFOS
//
// input:
//          dld_id         :  driver id
//
//          selection      :  empty pointer
//
// output:
//          selection      :  selected data channels
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_DATA_INFOS (DLD_ID dld_id, 
                   long channel, char* info_str, int len);



///////////////////////////////////////////////////////////
//
// DLD_GET_DATA_SELECTION
//
// input:
//          dld_id         :  driver id
//
//          selection      :  empty pointer
//
// output:
//          selection      :  selected data channels
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_DATA_SELECTION (DLD_ID dld_id, 
                   long* selection);



///////////////////////////////////////////////////////////
//
// DLD_SET_DATA_SELECTION
//
// input:
//          dld_id         :  driver id
//
//          selection      :  selected data channels
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_DATA_SELECTION (DLD_ID dld_id, 
                   long selection);



///////////////////////////////////////////////////////////
//
// DLD_GET_STATUS
//
// input:
//          dld_id         :  driver id
//          state          :  empty pointer
//
// output:
//          state          :  current device state
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_STATUS (DLD_ID dld_id, int* state);



///////////////////////////////////////////////////////////
//
// DLD_ADD_BUFFER
//
// input:
//          dld_id         :  driver id
//          dld_buf        :  prepared buffer
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_ADD_BUFFER (DLD_ID dld_id, dld_buf* buffer);




///////////////////////////////////////////////////////////
//
// DLD_REMOVE_BUFFER
//
// input:
//          dld_id         :  driver id
//          int            :  buffer id
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_REMOVE_BUFFER (DLD_ID dld_id, int buf_id);



///////////////////////////////////////////////////////////
//
// DLD_GET_BUFFER_STATUS
//
// input:
//          dld_id         :  driver id
//          int            :  buffer id
//          state          :  empty pointer
//
// output:
//          state          :  current device state
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_BUFFER_STATUS (DLD_ID dld_id, int buf_id, int* state);



///////////////////////////////////////////////////////////
//
// DLD_GET_MAX_BUFFERS
//
// input:
//          dld_id         :  driver id
//          num_buffers    :  empty pointer
//
// output:
//          num_buffers    :  max. number of buffers that can be hold
//                            in buffer queue
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_MAX_BUFFERS (DLD_ID dld_id, int* num_buffers);



///////////////////////////////////////////////////////////
//
// DLD_GET_EMPTY_BUFFER_NUM
//
// input:
//          dld_id         :  driver id
//          num_buffers    :  empty pointer
//
// output:
//          num_buffers    :  number of empty buffers in queue
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_EMPTY_BUFFER_NUM (DLD_ID dld_id, int* num_buffers);



///////////////////////////////////////////////////////////
//
// DLD_GET_FILLED_BUFFER_NUM
//
// input:
//          dld_id         :  driver id
//          num_buffers    :  empty pointer
//
// output:
//          num_buffers    :  number of filled buffers in queue
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_FILLED_BUFFER_NUM (DLD_ID dld_id, int* num_buffers);



///////////////////////////////////////////////////////////
//
// DLD_GET_NEXT_FILLED_BUFFER
//
// input:
//          dld_id         :  driver id
//          int            :  empty pointer
//
// output:
//          int            :  next filled buffer (id) in queue
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_NEXT_FILLED_BUFFER (DLD_ID dld_id, int* buf_id);



///////////////////////////////////////////////////////////
//
// DLD_RESET_BUFFER_QUEUE
//
// input:
//          dld_id         :  driver id
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_RESET_BUFFER_QUEUE (DLD_ID dld_id);



///////////////////////////////////////////////////////////
//
// DLD_GET_IMG_TRANFORMATION
//
// input:
//          dld_id         :  driver id
//
//          trans          :  pointer to empty image transformation table
//
// output:
//          trans          :  filled image transformation table
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_IMG_TRANFORMATION (DLD_ID dld_id, dld_img_transformation* trans);



///////////////////////////////////////////////////////////
//
// DLD_SET_IMG_TRANFORMATION
//
// input:
//          dld_id         :  driver id
//
//          trans          :  image transformation table
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_IMG_TRANFORMATION (DLD_ID dld_id, dld_img_transformation trans);



///////////////////////////////////////////////////////////
//
// acquisition methods
//

///////////////////////////////////////////////////////////
//
// DLD_START
//
// input:
//          dld_id         :  driver id
//          buf_event      :  HANDLE to event
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_START (DLD_ID dld_id, HANDLE buf_event);


///////////////////////////////////////////////////////////
//
// DLD_STOP
//
// input:
//          dld_id         :  driver id
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_STOP (DLD_ID dld_id);



///////////////////////////////////////////////////////////
//
// DLD_ABORT
//
// input:
//          dld_id         :  driver id
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_ABORT (DLD_ID dld_id);



///////////////////////////////////////////////////////////
//
// DLD_SUSPEND
//
// input:
//          dld_id         :  driver id
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SUSPEND (DLD_ID dld_id);



///////////////////////////////////////////////////////////
//
// DLD_RESUME
//
// input:
//          dld_id         :  driver id
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_RESUME (DLD_ID dld_id);





///////////////////////////////////////////////////////////
//
// debugging
//

///////////////////////////////////////////////////////////
//
// DLD_GET_DEBUG_LEVEL
//
// input:
//          level          :  empty pointer
//
// output:
//          level          :  debug level
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_GET_DEBUG_LEVEL (int* level);



///////////////////////////////////////////////////////////
//
// DLD_SET_DEBUG_LEVEL
//
// input:
//          level          :  debug level
//
// output:
//
// return:  ERR_NO or error code
//
DELAYLINE_API int DLD_SET_DEBUG_LEVEL (int level);




///////////////////////////////////////////////////////////
//
// DLD_SET_CNT
//
// input:
//			dld_id          :   driver id
//          cnt_time		:	counter time base
//			cnt_active		:	active counter channels 
//			cnt_delay		:	counter delay
//
// output:
//
// return:  ERR_NO or error code
DELAYLINE_API int DLD_SET_CNT (DLD_ID dld_id, long cnt_time, unsigned short cnt_active, long cnt_delay); 




///////////////////////////////////////////////////////////
//
// DLD_READY_CNT
//
// input:
//			dld_id          :   driver id
//
// output: 
//
// return:  ERR_NO, error code, number of CNT buffers
DELAYLINE_API int DLD_READY_CNT (DLD_ID dld_id);


///////////////////////////////////////////////////////////
//
// DLD_GET_CNT
//
// input:
//          dld_id          :   driver id
//
// output:
//			dat[9]			:   set of counterdata
//
// return:  ERR_NO or error code
DELAYLINE_API int DLD_GET_CNT (DLD_ID dld_id, long dat[9]);


///////////////////////////////////////////////////////////
//
// DLD_SET_CAL
//
// input:
//          dld_id          :   driver id
//			CalCoord		:	coordinates for calibrations
//			DefauftCoord	:	use defauft values for calibration coordinates
//			MakeIntCal		:	set switch for renewing intensity calibration file with the next exposure
//
// output:
//
// return:  ERR_NO or error code
DELAYLINE_API int DLD_SET_CAL (DLD_ID dld_id, dld_cal_region *CalCoord, bool DefaultCoord, bool MakeIntCal);


///////////////////////////////////////////////////////////
//
// DLD_SET_I2C_RES
//
// input:
//          dld_id          :   driver id
//			I2CAddress		:	address of element at I2C bus
//			ResistorNo		:	resistor address in the element (e.g. 0 or 1 for a 2 resistor element)
//			ResistorValue	:	resistor setting translated into the 8 bit number range 0..255 for Rmin..Rmax
//
// output:
//
// return:  ERR_NO or error code
DELAYLINE_API int DLD_SET_I2C_RES (DLD_ID dld_id, I2C_RT ResistorType, unsigned int I2CAddress, unsigned int ResistorNo, unsigned int ResistorValue);


///////////////////////////////////////////////////////////
//
// DLD_SET_QDLD_SHIFTS
//
// input:
//          dld_id          :   driver id
//			*Q_shifts			:	pointer to a 8 element array for shift_x1, shift_y1, shift_x2,.., shift_x4, shift_y4
//								for the 4 DLD quadrants 1..4 (display shifts for mapping the quadrants)
//
// output:
//
// return:  ERR_NO or error code
DELAYLINE_API int DLD_SET_QDLD_SHIFTS (DLD_ID dld_id, unsigned short * Q_shifts);


#ifdef __cplusplus
}
#endif  //C++

//
#endif // _DelayLine_h_

